@@ -31,8 +31,9 @@ import socket
 import decimal
 import datetime
 import time
+import collections
 
-VERSION = (0, 2, 7)
+VERSION = (0, 2, 8)
 __version__ = '%s.%s.%s' % VERSION
 apilevel = '2.0'
 threadsafety = 1
@@ -247,8 +248,8 @@ def _decode_column(data, oid, encoding):
         return b''.join([chr(c) for c in ia]) if PY2 else bytes(ia)
     elif oid in (PG_TYPE_TEXT, PG_TYPE_BPCHAR, PG_TYPE_VARCHAR, PG_TYPE_NAME, PG_TYPE_JSON):
         return data
-    elif oid in (PG_TYPE_UNKNOWN, ):
-        DEBUG_OUTPUT('UNKNOWN type')
+    elif oid in (PG_TYPE_UNKNOWN, PG_TYPE_PGNODETREE):
+        DEBUG_OUTPUT('NO DECODE type:', oid)
         return data
     else:
         if DEBUG:
@@ -296,7 +297,7 @@ def escape_parameter(v):
     elif t == bytearray or t == bytes:        # binary
         return "'" + ''.join(['\\%03o' % (c, ) for c in v]) + "'::bytea"
     elif t == bool:
-        return u"'t'" if v else u"'f'"
+        return u"TRUE" if v else u"FALSE"
     elif t == time.struct_time:
         return u'%04d-%02d-%02d %02d:%02d:%02d' % (
             v.tm_year, v.tm_mon, v.tm_mday, v.tm_hour, v.tm_min, v.tm_sec)
@@ -305,8 +306,10 @@ def escape_parameter(v):
             return u"interval '%d second'" % (v.days * 86400 + v.seconds, )
         else:
             return u"interval '%d day'" % (v.days, )
-    elif t == int or t == float or t == decimal.Decimal or (PY2 and t == long):
+    elif t == int or t == float or (PY2 and t == long):
         return str(v)
+    elif t == decimal.Decimal:
+        return "'" + str(v) + "'"
     else:
         return "'" + str(v) + "'"
 
@@ -383,8 +386,7 @@ class Cursor(object):
     def __init__(self, connection):
         self.connection = connection
         self.description = []
-        self._rows = []
-        self._current_row = -1
+        self._rows = collections.deque()
         self._rowcount = 0
         self.arraysize = 1
 
@@ -403,8 +405,7 @@ class Cursor(object):
     def execute(self, query, args=()):
         DEBUG_OUTPUT('Cursor::execute()', query, args)
         self.description = []
-        self._rows = []
-        self._current_row = -1
+        self._rows.clear()
         self.query = query
         self.args = args
         if args:
@@ -422,20 +423,21 @@ class Cursor(object):
         self._rowcount = rowcount
 
     def fetchone(self):
-        self._current_row += 1
-        DEBUG_OUTPUT('Cursor::fetchone()', self._current_row, self._rowcount)
-        if self._current_row >= self._rowcount:
-            return None
-        return self._rows[self._current_row]
+        DEBUG_OUTPUT('Cursor::fetchone()')
+        if len(self._rows):
+            return self._rows.popleft()
+        return None
 
     def fetchmany(self, size=1):
-        self._current_row += 1
-        r = self._rows[self._current_row:self._current_row+size]
-        self._current_row += size -1
+        r = []
+        for i in range(size):
+            r.append(r.fetchone())
         return r
 
     def fetchall(self):
-        return self._rows
+        r = list(self._rows)
+        self._rows.clear()
+        return r
 
     def close(self):
         self.connection = None
@@ -515,12 +517,10 @@ class Connection(object):
                 DEBUG_OUTPUT("COMMAND_COMPLETE:", command)
                 if command == 'SHOW':
                     obj._rowcount = 1
-                    obj._current_row = -1
                 else:
                     for k in ('SELECT', 'UPDATE', 'DELETE', 'INSERT'):
                         if command[:len(k)] == k:
                             obj._rowcount = int(command.split(' ')[-1])
-                            obj._current_row = -1
                             break
             elif code == PG_B_ROW_DESCRIPTION:
                 if not obj:
@@ -580,9 +580,11 @@ class Connection(object):
                 for b in err:
                     DEBUG_OUTPUT("\t\t", b.decode(self.encoding))
                 # http://www.postgresql.org/docs/9.3/static/errcodes-appendix.html
-                errcode = err[1][1:].decode(self.encoding)
-                message = errcode + u':' + err[2][1:].decode(self.encoding)
-                if errcode[:2] == u'23':
+                errcode = err[1][1:]
+                message = errcode + b':' + err[2][1:]
+                if not PY2:
+                    message = message.decode(self.encoding)
+                if errcode[:2] == b'23':
                     errobj = IntegrityError( message)
                 else:
                     errobj = DatabaseError(message)
